
#include "vtkVolume.h"
#include "vtkCamera.h"
#include "vtkImageGaussianSmooth.h"
#include "vtkWindowToImageFilter.h"
#include "WindowToImageFilter.h"
#include "vtkTIFFWriter.h"
#include "vtkJPEGWriter.h"

#include "vtkbox.h"


#define PI 3.14159265



VTKBox::VTKBox (QWidget *parent, const char *name)  : vtkQtRenderWindow (parent, name)
{
	background[0] = 0;
	background[1] = 0;
	background[2] = 0;
	Renderer = vtkRenderer::New();
	Renderer->SetBackground(background);
	volumeProperty = vtkVolumeProperty::New();
	volumeMapper = vtkVolumeRayCastMapper::New();
	volume = vtkVolume::New();
	compositeFunction = new VolumeRayCastCompositeFunction();
	this->AddRenderer(Renderer);
	this->Render();
	gradientRange = 1;
	dimension[0] = 0;
	dimension[1] = 0;
	dimension[2] = 0;
	spacing[0] = 0;
	spacing[1] = 0;
	spacing[2] = 0;
	ML_direction[0] = 0;
	ML_direction[1] = 0;
	ML_direction[2] = 1;
}


VTKBox::~VTKBox()
{
	this->RemoveRenderer(Renderer);
	this->Delete();
	Renderer->RemoveVolume(volume);
	volumeMapper->Delete();
	volumeProperty->Delete();
	compositeFunction->Delete();
	volume->Delete();
	Renderer->Delete();
}
	
	
void VTKBox::setRenderer(vtkImageData *imageData, vtkColorTransferFunction *colorFunc, vtkPiecewiseFunction *opacityFunc) 
{
    //setRenderStatus("Make Renderer");
	Renderer = vtkRenderer::New();
	Renderer->SetBackground(background);
	volumeProperty = vtkVolumeProperty::New();
	volumeMapper = vtkVolumeRayCastMapper::New();
	volume = vtkVolume::New();
	compositeFunction = new VolumeRayCastCompositeFunction();
	vtkRenderer *renderer = makeRenderer(imageData, colorFunc, opacityFunc);
    if (!renderer)
		return;
    if (Renderer)
		this->RemoveRenderer(Renderer);
    Renderer = renderer;
    this->AddRenderer(Renderer);
    this->SetFDFrames(0);
    //this->Render();
	//setRenderStatus("");
}


vtkRenderer* VTKBox::makeRenderer(vtkImageData *imageData, vtkColorTransferFunction *cTFun, vtkPiecewiseFunction *oTFun) 
{   
    vtkRenderer *renderer = vtkRenderer::New();
	renderer->SetBackground(background);
	renderer->BackingStoreOn();

    volumeProperty->SetColor(cTFun);
    volumeProperty->SetScalarOpacity(oTFun);
    volumeProperty->SetInterpolationTypeToLinear();
    volumeProperty->ShadeOn();

    // Create the volume mapper and set the ray function and scalar input
    volumeMapper->SetVolumeRayCastFunction(compositeFunction);
    volumeMapper->SetInput(imageData);

	gradientRange = imageData->GetScalarTypeMax();
	if (gradientRange > 4096)
		gradientRange = 4096;

	imageData->GetDimensions(dimension);
	imageData->GetSpacing(spacing);
	float origin [3];
	origin[0] = (float)(dimension[0]/2)*spacing[0];
	origin[1] = (float)(dimension[1]/2)*spacing[1];
	origin[2] = (float)(dimension[2]/2)*spacing[2];

    // Create the volume and set the mapper and property
    volume->SetMapper(volumeMapper);
    volume->SetProperty(volumeProperty);
	volume->SetOrigin(origin);
    
    renderer->AddVolume(volume);

    return renderer;
}


void VTKBox::repaintTF(vtkColorTransferFunction *colorFunc, vtkPiecewiseFunction *opacityFunc)
{
    setRenderStatus("Repaint Transfer Function");
    volumeProperty->SetColor(colorFunc);
    volumeProperty->SetScalarOpacity(opacityFunc);
	if (right_rot)
	{
		volume->RotateX(0.0000000000000000001);
		right_rot = false;
	}
	else
	{
		volume->RotateX(-0.0000000000000000001);
		right_rot = true;
	}
    this->Render();
	setRenderStatus("");
}


float VTKBox::getGradientRange()
{
    return gradientRange;
}


double* VTKBox::getCamPos()
{
    double *camPos = new double [3];
    Renderer->GetActiveCamera()->GetPosition(camPos);
    return camPos;
}


float* VTKBox::getOrientation()
{
	float *orientation = new float [3];
	volume->GetOrientation(orientation);
	return orientation;
}


double* VTKBox::getFocalPoint()
{
    double *focPoint = new double [3];
    Renderer->GetActiveCamera()->GetFocalPoint(focPoint);
    return focPoint;
}


void VTKBox::setFocalPoint(double *focPoint)
{
	setRenderStatus("Setting Focal Point");
	float cameraPos [3];
	Renderer->GetActiveCamera()->GetPosition(cameraPos);
	cameraPos[0] = focPoint[0];
	cameraPos[1] = focPoint[1];
	Renderer->GetActiveCamera()->SetPosition(cameraPos);
    Renderer->GetActiveCamera()->SetFocalPoint(focPoint);
	if (right_rot)
	{
		volume->RotateX(0.0000000000000000001);
		right_rot = false;
	}
	else
	{
		volume->RotateX(-0.0000000000000000001);
		right_rot = true;
	}
	this->Render();
	setRenderStatus("");
}


void VTKBox::rendering(int xRot, int yRot, int zRot, double zoom, float sampleStep, bool parallelProj)
{
	float *orientation = new float [3];
	volume->GetOrientation(orientation);
	orientation[0] += xRot;
	orientation[1] += yRot;
	orientation[2] += zRot;
	volume->SetOrientation(orientation);
	float scale [3];
	scale[0] = zoom;
	scale[1] = zoom;
	scale[2] = zoom;
	volume->SetScale(scale);
	if (parallelProj)
		Renderer->GetActiveCamera()->ParallelProjectionOn();
	else
		Renderer->GetActiveCamera()->ParallelProjectionOff();
	volumeMapper->SetSampleDistance(sampleStep);
}


void VTKBox::rendering(vtkPiecewiseFunction *gradientFunc)
{
	//setRenderStatus("Rendering Image");
	volumeProperty->SetGradientOpacity(gradientFunc);
	//setRenderStatus("");
}

void VTKBox::setBackground(float backgr [3])
{
	background[0] = backgr[0];
	background[1] = backgr[1];
	background[2] = backgr[2];
	Renderer->SetBackground(background);
}


float * VTKBox::getBackground()
{
	return background;
}


void VTKBox::RenderImage()
{
	setRenderStatus("Rendering Image");
	if (right_rot)
	{
		volume->RotateX(0.0000000000000000001);
		right_rot = false;
	}
	else
	{
		volume->RotateX(-0.0000000000000000001);
		right_rot = true;
	}
	if (compositeFunction->getMagicLampStatus())
	{
		float *orientation = new float [3];
		float lampDirection [3];
		volume->GetOrientation(orientation);
		lampDirection[0] = ML_direction[0];
		lampDirection[1] = ML_direction[1];
		lampDirection[2] = ML_direction[2];
		float help [3];
		help[0] = lampDirection[0]*cos((-1)*PI/180*orientation[2]) - lampDirection[1]*sin((-1)*PI/180*orientation[2]);
		help[1] = lampDirection[0]*sin((-1)*PI/180*orientation[2]) + lampDirection[1]*cos((-1)*PI/180*orientation[2]);
		lampDirection[0] = help[0];
		lampDirection[1] = help[1];
		help[2] = lampDirection[2]*cos((-1)*PI/180*orientation[1]) - lampDirection[0]*sin((-1)*PI/180*orientation[1]);
		help[0] = lampDirection[2]*sin((-1)*PI/180*orientation[1]) + lampDirection[0]*cos((-1)*PI/180*orientation[1]);
		lampDirection[0] = help[0];
		lampDirection[2] = help[2];
		help[1] = lampDirection[1]*cos((-1)*PI/180*orientation[0]) - lampDirection[2]*sin((-1)*PI/180*orientation[0]);
		help[2] = lampDirection[1]*sin((-1)*PI/180*orientation[0]) + lampDirection[2]*cos((-1)*PI/180*orientation[0]);
		lampDirection[1] = help[1];
		lampDirection[2] = help[2];
		compositeFunction->setMagicLampDirection(lampDirection);
	}
	this->Render();
	setRenderStatus("");
}


void VTKBox::setFocalDisk(double focalDisk)
{
    Renderer->GetActiveCamera()->SetFocalDisk(focalDisk);
}


void VTKBox::setSamplePoints(int value)
{
    FDFrames = value;
}


void VTKBox::renderingDOF(bool random)
{
	Renderer->BackingStoreOff();
	if (random)
	{
		setRenderStatus("Rendering Depth of Field");
		this->SetFDFrames(FDFrames);
		this->Render();
		this->SetFDFrames(0);
		setRenderStatus("");
	}
	else
	{
		setRenderStatus("Rendering Depth of Field");
		double focalDisc = Renderer->GetActiveCamera()->GetFocalDisk();
		double angleStep = 2*PI/FDFrames;
		float startPosition [3];
		float focalPoint [3];
		Renderer->GetActiveCamera()->GetPosition(startPosition);
		Renderer->GetActiveCamera()->GetFocalPoint(focalPoint);
		double distance = sqrt((startPosition[0]-focalPoint[0])*(startPosition[0]-focalPoint[0])
							+ (startPosition[1]-focalPoint[1])*(startPosition[1]-focalPoint[1])
							+ (startPosition[2]-focalPoint[2])*(startPosition[2]-focalPoint[2]));
		double angle = tan((focalDisc/2)/distance)*180/PI;
		double currentAngle = 0;
		this->SetSubFrames(FDFrames+1);
		this->Render();
		for (int i = 0; i < FDFrames; i++)
		{
			Renderer->GetActiveCamera()->Azimuth(angle*cos(currentAngle));
			Renderer->GetActiveCamera()->Elevation(angle*sin(currentAngle));
			setRenderStatus("Render Depth of Field " + QString::number(i+1) + "/" + QString::number(FDFrames));
			this->Render();
			Renderer->GetActiveCamera()->Elevation(-angle*sin(currentAngle));
			Renderer->GetActiveCamera()->Azimuth(-angle*cos(currentAngle));
			currentAngle = currentAngle+angleStep;
		}
		this->SetSubFrames(0);
		setRenderStatus("");
	}
	Renderer->BackingStoreOn();
}


void VTKBox::renderingDOF(double focalDisc, int samplePoints)
{
	double angleStep = 2*PI/samplePoints;
	float startPosition [3];
	float focalPoint [3];
	Renderer->GetActiveCamera()->GetPosition(startPosition);
	Renderer->GetActiveCamera()->GetFocalPoint(focalPoint);
	double distance = sqrt((startPosition[0]-focalPoint[0])*(startPosition[0]-focalPoint[0])
						+ (startPosition[1]-focalPoint[1])*(startPosition[1]-focalPoint[1])
						+ (startPosition[2]-focalPoint[2])*(startPosition[2]-focalPoint[2]));
	double angle = tan((focalDisc/2)/distance)*180/PI;
	double currentAngle = 0;
	this->SetSubFrames(samplePoints+1);
	this->Render();
	for (int i = 0; i < samplePoints; i++)
	{
		Renderer->GetActiveCamera()->Azimuth(angle*cos(currentAngle));
		Renderer->GetActiveCamera()->Elevation(angle*sin(currentAngle));
		this->Render();
		Renderer->GetActiveCamera()->Elevation(-angle*sin(currentAngle));
		Renderer->GetActiveCamera()->Azimuth(-angle*cos(currentAngle));
		currentAngle = currentAngle+angleStep;
	}
	this->SetSubFrames(0);
}


void VTKBox::saveImage(QString file, bool DOF, bool DOF_random)
{
	setRenderStatus("saving Image");
	if (DOF)
		renderingDOF(DOF_random);
	else
	{
		if (right_rot)
		{
			volume->RotateX(0.0000000000000000001);
			right_rot = false;
		}
		else
		{
			volume->RotateX(-0.0000000000000000001);
			right_rot = true;
		}
		this->Render();
	}
	setRenderStatus("saving Image");
	WindowToImageFilter *w2i = new WindowToImageFilter();
    w2i->SetInput(this);
	w2i->ReadFrontBufferOn();
	
	vtkTIFFWriter *writer = vtkTIFFWriter::New();
    writer->SetInput(w2i->GetOutput());
    writer->SetFileName(file);
    writer->Write();

	w2i->Delete();
	writer->Delete();
    
    setRenderStatus("");
}


void VTKBox::startDOFAnimation(double points [20][3], int number, float start, float end, int frames, int samplePoints, bool fixedPoints, QString directory)
{
    setRenderStatus("Rendering Animation");
	/*points[0][0] = 58.9;
	points[0][1] = 170.5;
	points[0][2] = 380;
	points[1][0] = 58.9;
	points[1][1] = 170.5;
	points[1][2] = 0;
	number = 2;
	start = 39.3;
	end = 39.3;
	frames = 90;
	samplePoints = 16;
	fixedPoints = true;*/
	Renderer->BackingStoreOff();
	bool FC = false;
	if (compositeFunction->getFocusContextStatus())
	{
		FC = true;
		//compositeFunction->setFocusContextDisable();
	}

	FILE *file;
	bool writeFile = true;
	file = fopen (directory+"/animationSteps.txt", "wt+");
	if (file == NULL)
	{
		writeFile = false;
	}

	double *startFocPoint = new double [3];
	float camPos [3];
    Renderer->GetActiveCamera()->GetFocalPoint(startFocPoint);
	double *length = new double [number-1];
    double abs_length = 0;
    double step;
    int *numberSteps = new int [number-1];
	int i;
    double focPoint [3];
    float lenseStep = (end-start)/(float)(frames);
    float lenseThickness = start;

    for (i = 0; i < number-1; i ++)
    {
		length[i] = sqrt((points[i][0]-points[i+1][0])*(points[i][0]-points[i+1][0]) + (points[i][1]-points[i+1][1])*(points[i][1]-points[i+1][1]) + (points[i][2]-points[i+1][2])*(points[i][2]-points[i+1][2]));
    }
    for (i = 0; i < number-1; i++)
    {
		abs_length = abs_length + length[i];
    }
    step = abs_length/(double)(frames);
    for (i = 0; i < number-1; i++)
    {
		if (step != 0)
		{
			numberSteps[i] = (int)(length[i]/step);
		}
		else
		{
			numberSteps[i] = (int)frames/(number-1);
		}
    }

	float scale [3];
	float *orientation;
	volume->GetScale(scale);
	orientation = volume->GetOrientationWXYZ();
	QString initialValues = "xRot = " + QString::number(orientation[1]) +
		", yRot = " + QString::number(orientation[2]) +
		", zRot = " + QString::number(orientation[3]) +
		", zoom = " + QString::number(scale[0]) + 
		", samplePoints = " + QString::number(samplePoints) + 
		", frames = " + QString::number(frames) + "\n";
	if (writeFile)
		fputs(initialValues, file);

    int k = 0;
	focPoint[0] = points[0][0];
	focPoint[1] = points[0][1];
	focPoint[2] = points[0][2];
    for (i = 0; i < number-1; i++)
    {
		double stepx = (double)(points[i+1][0] - points[i][0])/numberSteps[i];
		double stepy = (double)(points[i+1][1] - points[i][1])/numberSteps[i];
		double stepz = (double)(points[i+1][2] - points[i][2])/numberSteps[i];

		QString point = "Point Nr. " + QString::number(i) + " [" + QString::number(numberSteps[i]) + " steps]\n";
		if (writeFile)
			fputs(point, file);

		for (int j = 0; j <= numberSteps[i]; j ++)
		{
			QString picName;
			if (k < 10)
				picName = "pic00" + QString::number(k) + ".jpg";
			else if (k < 100)
				picName = "pic0" + QString::number(k) + ".jpg";
			else
				picName = "pic" + QString::number(k) + ".jpg";

			QString pointDatas = "Nr. " + QString::number(k) + ": " +
				"Focal Point (" + QString::number(focPoint[0]) + ", " + QString::number(focPoint[1]) + ", " + QString::number(focPoint[2]) + ")" +
				", LenseThickness (" + QString::number(lenseThickness) + ")\n";
			if (writeFile)
				fputs(pointDatas, file);
			
			//rendern
			QString filename = directory + "/" + picName;
			k++;

			setRenderStatus("Rendering Animation " + QString::number(k) + "/" + QString::number(frames));

			Renderer->GetActiveCamera()->SetFocalPoint(focPoint);
			Renderer->GetActiveCamera()->GetPosition(camPos);
			camPos[0] = focPoint[0];
			camPos[1] = focPoint[1];
			Renderer->GetActiveCamera()->SetPosition(camPos);

			renderingDOF(lenseThickness, samplePoints);

			// save Image
			WindowToImageFilter *w2i = new WindowToImageFilter();
			w2i->SetInput(this);
    
			vtkJPEGWriter *writer = vtkJPEGWriter::New();
			writer->SetInput(w2i->GetOutput());
			writer->SetFileName(filename);
			writer->Write();

			writer->Delete();
			w2i->Delete();

	    
			if (!fixedPoints)
			{
				focPoint[0] = focPoint[0] + stepx;
				focPoint[1] = focPoint[1] + stepy;
			}
			focPoint[2] = focPoint[2] + stepz;
			lenseThickness = lenseThickness + lenseStep;
		} 
    }  
	delete length;
	delete numberSteps;

	fclose(file);
	
	if (FC)
		compositeFunction->setFocusContextEnable();
	setFocalPoint(startFocPoint);
	Renderer->GetActiveCamera()->GetPosition(camPos);
	camPos[0] = startFocPoint[0];
	camPos[1] = startFocPoint[1];
	Renderer->GetActiveCamera()->SetPosition(camPos);
	Renderer->BackingStoreOn();
	if (right_rot)
	{
		volume->RotateX(0.0000000000000000001);
		right_rot = false;
	}
	else
	{
		volume->RotateX(-0.0000000000000000001);
		right_rot = true;
	}
	this->Render();
	setRenderStatus("");
}


void VTKBox::renderingFC(float * middle, float radius)
{
	compositeFunction->setFocusArea(middle, radius);
}


void VTKBox::renderingFC(vtkColorTransferFunction *cTF, vtkPiecewiseFunction *pwF, float transC, float ksc, float kss, float kse, float redOffset, float redFactor)
{
	compositeFunction->setContextVolume(cTF, pwF);
	compositeFunction->setParameters(transC, ksc, kss, kse, redOffset, redFactor);
}


void VTKBox::setFocusContext(bool status)
{
	if (status)
		compositeFunction->setFocusContextEnable();
	else
		compositeFunction->setFocusContextDisable();
}


void VTKBox::startFCAnimation(FCAnim_struct *animPoints, int number, QString directory, bool status)
{
	setRenderStatus("Rendering F+C Animation");
	Renderer->BackingStoreOff();

	float *defaultOrientation = volume->GetOrientation();
	float *defaultScale = volume->GetScale();
	if (!status)
		compositeFunction->setFocusContextEnable();

	FILE *file;
	bool writeFile = true;
	file = fopen (directory+"/animationSteps.txt", "wt+");
	if (file == NULL)
	{
		writeFile = false;
	}	

	float frames;
	int frameCounter = 0;
	int overallFrames = 0;
	for (int k = 1; k < number; k++)
		overallFrames += animPoints[k].samples;

	float middle [3];
	float radius;
	float transfC;
	float ksc;
	float kss;
	float kse;
	float redOffset;
	float redFactor;
	float xRot;
	float yRot;
	float zRot;
	float zoom;
	
	float middle_diff [3];
	float radius_diff;
	float transfC_diff;
	float ksc_diff;
	float kss_diff;
	float kse_diff;
	float redOffset_diff;
	float redFactor_diff;
	float xRot_diff;
	float yRot_diff;
	float zRot_diff;
	float zoom_diff;

	double middle_step [3];
	double radius_step;
	double transfC_step;
	double ksc_step;
	double kss_step;
	double kse_step;
	double redOffset_step;
	double redFactor_step;
	double xRot_step;
	double yRot_step;
	double zRot_step;
	double zoom_step;

	// do something !!!!
	for (int i = 1; i < number; i++)
	{
		// calculate Differences
		middle_diff[0] = animPoints[i].center[0] - animPoints[i-1].center[0];
		middle_diff[1] = animPoints[i].center[1] - animPoints[i-1].center[1];
		middle_diff[2] = animPoints[i].center[2] - animPoints[i-1].center[2];
		radius_diff = animPoints[i].radius - animPoints[i-1].radius;
		transfC_diff = animPoints[i].transfC - animPoints[i-1].transfC;
		ksc_diff = animPoints[i].ksc - animPoints[i-1].ksc;
		kss_diff = animPoints[i].kss - animPoints[i-1].kss;
		kse_diff = animPoints[i].kse - animPoints[i-1].kse;
		redOffset_diff = animPoints[i].redOffset - animPoints[i-1].redOffset;
		redFactor_diff = animPoints[i].redFactor - animPoints[i-1].redFactor;
		xRot_diff = animPoints[i].xRot - animPoints[i-1].xRot;
		if (xRot_diff >= 0)
		{
			if (!animPoints[i].xClockwise)
				xRot_diff = 360 - xRot_diff;
		}
		else
		{
			if (animPoints[i].xClockwise)
				xRot_diff = 360 + xRot_diff;
		}
		if (xRot_diff == 360)
			xRot_diff = 0;
		yRot_diff = animPoints[i].yRot - animPoints[i-1].yRot;
		if (yRot_diff >= 0)
		{
			if (!animPoints[i].yClockwise)
				yRot_diff = 360 - yRot_diff;
		}
		else
		{
			if (animPoints[i].yClockwise)
				yRot_diff = 360 + yRot_diff;
		}
		if (yRot_diff == 360)
			yRot_diff = 0;
		zRot_diff = animPoints[i].zRot - animPoints[i-1].zRot;
		if (zRot_diff >= 0)
		{
			if (!animPoints[i].zClockwise)
				zRot_diff = 360 - zRot_diff;
		}
		else
		{
			if (animPoints[i].zClockwise)
				zRot_diff = 360 + zRot_diff;
		}
		if (zRot_diff == 360)
			zRot_diff = 0;
		zoom_diff = animPoints[i].zoom - animPoints[i-1].zoom;

		// Startvalues
		middle[0] = animPoints[i-1].center[0];
		middle[1] = animPoints[i-1].center[1];
		middle[2] = animPoints[i-1].center[2];
		radius = animPoints[i-1].radius;
		transfC = animPoints[i-1].transfC;
		ksc = animPoints[i-1].ksc;
		kss = animPoints[i-1].kss;
		kse = animPoints[i-1].kse;
		redOffset = animPoints[i-1].redOffset;
		redFactor = animPoints[i-1].redFactor;
		xRot = animPoints[i-1].xRot;
		yRot = animPoints[i-1].yRot;
		zRot = animPoints[i-1].zRot;
		zoom = animPoints[i-1].zoom;

		// calculate Step per frame
		frames = animPoints[i].samples;

		middle_step[0] = middle_diff[0]/frames;
		middle_step[1] = middle_diff[1]/frames;
		middle_step[2] = middle_diff[2]/frames;
		radius_step = radius_diff/frames;
		transfC_step = transfC_diff/frames;
		ksc_step = ksc_diff/frames;
		kss_step = kss_diff/frames;
		kse_step = kse_diff/frames;
		redOffset_step = redOffset_diff/frames;
		redFactor_step = redFactor_diff/frames;
		xRot_step = xRot_diff/frames;
		yRot_step = yRot_diff/frames;
		zRot_step = zRot_diff/frames;
		zoom_step = zoom_diff/frames;

		QString point = "Point Nr. " + QString::number(i) + " [" + QString::number(animPoints[i].samples) + " frames]\n";
		if (writeFile)
			fputs(point, file);

		for (int j = 0; j < frames; j++)
		{
			setRenderStatus("Rendering F+C Animation " + QString::number(frameCounter+1) + "/" + QString::number(overallFrames));
			QString pointDatas = "   Frame Nr. " + QString::number(frameCounter) + ": " +
				"middle (" + QString::number(middle[0]) + ", " + QString::number(middle[1]) + ", " + QString::number(middle[2]) + "), " +
				"radius (" + QString::number(radius) + "), " + 
				"transfC (" + QString::number(transfC) + "), " +
				"ksc (" + QString::number(ksc) + "), " +
				"kss (" + QString::number(kss) + "), " +
				"kse (" + QString::number(kse) + "), " +
				"redOffset (" + QString::number(redOffset) + "), " +
				"redFactor (" + QString::number(redFactor) + "), " +
				"xRot (" + QString::number(xRot) + "), " +
				"yRot (" + QString::number(yRot) + "), " +
				"zRot (" + QString::number(zRot) + "), " +
				"zoom (" + QString::number(zoom) + ")\n";
			if (writeFile)
				fputs(pointDatas, file);

			QString picName;
			if (frameCounter < 10)
				picName = "pic000" + QString::number(frameCounter) + ".jpg";
			else if (frameCounter < 100)
				picName = "pic00" + QString::number(frameCounter) + ".jpg";
			else if (frameCounter < 1000)
				picName = "pic0" + QString::number(frameCounter) + ".jpg";
			else
				picName = "pic" + QString::number(frameCounter) + ".jpg";
			QString filename = directory + "/" + picName;



			// do rendering Step
			compositeFunction->setFocusArea(middle, radius);
			compositeFunction->setParameters(transfC, ksc, kss, kse, redOffset, redFactor);
			float * orientation = new float [3];
			volume->GetOrientation(orientation);
			orientation[0] += xRot_step;
			orientation[1] += yRot_step;
			orientation[2] += zRot_step;
			volume->SetOrientation(orientation);
			//volume->RotateX(xRot_step);
			//volume->RotateY(yRot_step);
			//volume->RotateZ(zRot_step);
			volume->SetScale(zoom, zoom, zoom);

			this->Render();


			// save Image
			WindowToImageFilter *w2i = new WindowToImageFilter();
			w2i->SetInput(this);
    
			vtkJPEGWriter *writer = vtkJPEGWriter::New();
			writer->SetInput(w2i->GetOutput ());
			writer->SetFileName(filename);
			writer->Write();

			writer->Delete();
			w2i->Delete();



			// increment values
			middle[0] += middle_step[0];
			middle[1] += middle_step[1];
			middle[2] += middle_step[2];
			radius += radius_step;
			transfC += transfC_step;
			ksc += ksc_step;
			kss += kss_step;
			kse += kse_step;
			redOffset += redOffset_step;
			redFactor += redFactor_step;
			xRot += xRot_step;
			yRot += yRot_step;
			zRot += zRot_step;
			zoom += zoom_step;

			frameCounter += 1;
		}
	}

	fclose(file);

	if (status)
		compositeFunction->setFocusContextEnable();
	else
		compositeFunction->setFocusContextDisable();
	volume->SetOrientation(defaultOrientation);
	volume->SetScale(defaultScale);
	Renderer->BackingStoreOn();
	this->Render();
	setRenderStatus("");
}


void VTKBox::setDitheringParameters(float distance, float size, bool context, bool structure, bool second_row, bool second_row_structure)
{
	compositeFunction->setDithering(dimension, spacing, distance, size, context, structure, second_row, second_row_structure);
}


void VTKBox::setDithering(bool status)
{
	if (status)
		compositeFunction->setDitheringEnable();
	else
		compositeFunction->setDitheringDisable();
}


void VTKBox::setMagicLampParameters(float direction [3], float angle, float transfer_factor, bool backside)
{
	compositeFunction->setMagicLamp(direction, angle, transfer_factor, backside);
	ML_direction[0] = direction[0];
	ML_direction[1] = direction[1];
	ML_direction[2] = direction[2];
}


void VTKBox::setMagicLamp(bool status)
{
	if (status)
		compositeFunction->setMagicLampEnable();
	else
		compositeFunction->setMagicLampDisable();
}


void VTKBox::startDithAnimation(DithAnim_struct *animPoints, int number, QString directory, bool status)
{
	setRenderStatus("Rendering 3D Dithering Animation");
	Renderer->BackingStoreOff();

	float *defaultOrientation = volume->GetOrientation();
	float *defaultScale = volume->GetScale();
	if (!status)
		compositeFunction->setDitheringEnable();

	FILE *file;
	bool writeFile = true;
	file = fopen (directory+"/animationSteps.txt", "wt+");
	if (file == NULL)
	{
		writeFile = false;
	}	

	float frames;
	int frameCounter = 0;
	int overallFrames = 0;
	for (int k = 1; k < number; k++)
		overallFrames += animPoints[k].samples;
	
	bool context = animPoints[0].context;
	bool structure = animPoints[0].structure;
	bool sec_row = animPoints[0].sec_row;
	bool sec_row_structure = animPoints[0].sec_row_structure;

	compositeFunction->setDithering(context, structure, sec_row, sec_row_structure);

	float distance;
	float size;
	float xRot;
	float yRot;
	float zRot;
	float zoom;
	
	float distance_diff;
	float size_diff;
	float xRot_diff;
	float yRot_diff;
	float zRot_diff;
	float zoom_diff;

	double distance_step;
	double size_step;
	double xRot_step;
	double yRot_step;
	double zRot_step;
	double zoom_step;

	// do something !!!!
	for (int i = 1; i < number; i++)
	{
		// calculate Differences
		distance_diff = animPoints[i].distance - animPoints[i-1].distance;
		size_diff = animPoints[i].size - animPoints[i-1].size;
		xRot_diff = animPoints[i].xRot - animPoints[i-1].xRot;
		if (xRot_diff >= 0)
		{
			if (!animPoints[i].xClockwise)
				xRot_diff = 360 - xRot_diff;
		}
		else
		{
			if (animPoints[i].xClockwise)
				xRot_diff = 360 + xRot_diff;
		}
		if (xRot_diff == 360)
			xRot_diff = 0;
		yRot_diff = animPoints[i].yRot - animPoints[i-1].yRot;
		if (yRot_diff >= 0)
		{
			if (!animPoints[i].yClockwise)
				yRot_diff = 360 - yRot_diff;
		}
		else
		{
			if (animPoints[i].yClockwise)
				yRot_diff = 360 + yRot_diff;
		}
		if (yRot_diff == 360)
			yRot_diff = 0;
		zRot_diff = animPoints[i].zRot - animPoints[i-1].zRot;
		if (zRot_diff >= 0)
		{
			if (!animPoints[i].zClockwise)
				zRot_diff = 360 - zRot_diff;
		}
		else
		{
			if (animPoints[i].zClockwise)
				zRot_diff = 360 + zRot_diff;
		}
		if (zRot_diff == 360)
			zRot_diff = 0;
		zoom_diff = animPoints[i].zoom - animPoints[i-1].zoom;

		// Startvalues
		distance = animPoints[i-1].distance;
		size = animPoints[i-1].size;
		xRot = animPoints[i-1].xRot;
		yRot = animPoints[i-1].yRot;
		zRot = animPoints[i-1].zRot;
		zoom = animPoints[i-1].zoom;

		// calculate Step per frame
		frames = animPoints[i].samples;

		distance_step = distance_diff/frames;
		size_step = size_diff/frames;
		xRot_step = xRot_diff/frames;
		yRot_step = yRot_diff/frames;
		zRot_step = zRot_diff/frames;
		zoom_step = zoom_diff/frames;

		QString point = "Point Nr. " + QString::number(i) + " [" + QString::number(animPoints[i].samples) + " frames]\n";
		if (writeFile)
			fputs(point, file);

		for (int j = 0; j < frames; j++)
		{
			setRenderStatus("Rendering 3D Dithering Animation " + QString::number(frameCounter+1) + "/" + QString::number(overallFrames));
			QString pointDatas = "   Frame Nr. " + QString::number(frameCounter) + ": " +
				"distance (" + QString::number(distance) + ", " +
				"size (" + QString::number(size) + "), " + 
				"xRot (" + QString::number(xRot) + "), " +
				"yRot (" + QString::number(yRot) + "), " +
				"zRot (" + QString::number(zRot) + "), " +
				"zoom (" + QString::number(zoom) + ")\n";
			if (writeFile)
				fputs(pointDatas, file);

			QString picName;
			if (frameCounter < 10)
				picName = "pic000" + QString::number(frameCounter) + ".jpg";
			else if (frameCounter < 100)
				picName = "pic00" + QString::number(frameCounter) + ".jpg";
			else if (frameCounter < 1000)
				picName = "pic0" + QString::number(frameCounter) + ".jpg";
			else
				picName = "pic" + QString::number(frameCounter) + ".jpg";
			QString filename = directory + "/" + picName;



			// do rendering Step
			if (distance_step != 0 || size_step != 0)
				compositeFunction->setDithering(dimension, spacing, distance, size, context, structure, sec_row, sec_row_structure);
			float * orientation = new float [3];
			volume->GetOrientation(orientation);
			orientation[0] += xRot_step;
			orientation[1] += yRot_step;
			orientation[2] += zRot_step;
			volume->SetOrientation(orientation);
			volume->SetScale(zoom, zoom, zoom);

			
			this->Render();

			// save Image
			WindowToImageFilter *w2i = new WindowToImageFilter();
			w2i->SetInput(this);
    
			vtkJPEGWriter *writer = vtkJPEGWriter::New();
			writer->SetInput(w2i->GetOutput ());
			writer->SetFileName(filename);
			writer->Write();

			writer->Delete();
			w2i->Delete();



			// increment values
			distance += distance_step;
			size += size_step;
			xRot += xRot_step;
			yRot += yRot_step;
			zRot += zRot_step;
			zoom += zoom_step;

			frameCounter += 1;
		}
	}

	fclose(file);

	if (status)
		compositeFunction->setDitheringEnable();
	else
		compositeFunction->setDitheringDisable();
	volume->SetOrientation(defaultOrientation);
	volume->SetScale(defaultScale);
	Renderer->BackingStoreOn();
	setRenderStatus("");
}


void VTKBox::startMLAnimation(MLAnim_struct *animPoints, int number, QString directory, bool status)
{
	setRenderStatus("Rendering Magic Lamp Animation");
	Renderer->BackingStoreOff();

	float *defaultOrientation = volume->GetOrientation();
	float *defaultScale = volume->GetScale();
	if (!status)
		compositeFunction->setMagicLampEnable();

	FILE *file;
	bool writeFile = true;
	file = fopen (directory+"/animationSteps.txt", "wt+");
	if (file == NULL)
	{
		writeFile = false;
	}	

	float frames;
	int frameCounter = 0;
	int overallFrames = 0;
	for (int k = 1; k < number; k++)
		overallFrames += animPoints[k].samples;

	bool backside = animPoints[0].backside;

	float direction [3];
	float angle;
	float transfer;
	float xRot;
	float yRot;
	float zRot;
	float zoom;
	
	float direction_diff [3];
	float angle_diff;
	float transfer_diff;
	float xRot_diff;
	float yRot_diff;
	float zRot_diff;
	float zoom_diff;

	double direction_step [3];
	double angle_step;
	double transfer_step;
	double xRot_step;
	double yRot_step;
	double zRot_step;
	double zoom_step;

	// do something !!!!
	for (int i = 1; i < number; i++)
	{
		// calculate Differences
		direction_diff[0] = animPoints[i].direction[0] - animPoints[i-1].direction[0];
		direction_diff[1] = animPoints[i].direction[1] - animPoints[i-1].direction[1];
		direction_diff[2] = animPoints[i].direction[2] - animPoints[i-1].direction[2];
		angle_diff = animPoints[i].angle_diff - animPoints[i-1].angle_diff;
		transfer_diff = animPoints[i].transfer - animPoints[i-1].transfer;
		xRot_diff = animPoints[i].xRot - animPoints[i-1].xRot;
		if (xRot_diff >= 0)
		{
			if (!animPoints[i].xClockwise)
				xRot_diff = 360 - xRot_diff;
		}
		else
		{
			if (animPoints[i].xClockwise)
				xRot_diff = 360 + xRot_diff;
		}
		if (xRot_diff == 360)
			xRot_diff = 0;
		yRot_diff = animPoints[i].yRot - animPoints[i-1].yRot;
		if (yRot_diff >= 0)
		{
			if (!animPoints[i].yClockwise)
				yRot_diff = 360 - yRot_diff;
		}
		else
		{
			if (animPoints[i].yClockwise)
				yRot_diff = 360 + yRot_diff;
		}
		if (yRot_diff == 360)
			yRot_diff = 0;
		zRot_diff = animPoints[i].zRot - animPoints[i-1].zRot;
		if (zRot_diff >= 0)
		{
			if (!animPoints[i].zClockwise)
				zRot_diff = 360 - zRot_diff;
		}
		else
		{
			if (animPoints[i].zClockwise)
				zRot_diff = 360 + zRot_diff;
		}
		if (zRot_diff == 360)
			zRot_diff = 0;
		zoom_diff = animPoints[i].zoom - animPoints[i-1].zoom;

		// Startvalues
		direction[0] = animPoints[i-1].direction[0];
		direction[1] = animPoints[i-1].direction[1];
		direction[2] = animPoints[i-1].direction[2];
		angle = animPoints[i-1].angle_diff;
		transfer = animPoints[i-1].transfer;
		xRot = animPoints[i-1].xRot;
		yRot = animPoints[i-1].yRot;
		zRot = animPoints[i-1].zRot;
		zoom = animPoints[i-1].zoom;

		// calculate Step per frame
		frames = animPoints[i].samples;

		direction_step[0] = direction_diff[0]/frames;
		direction_step[1] = direction_diff[1]/frames;
		direction_step[2] = direction_diff[2]/frames;
		angle_step = angle_diff/frames;
		transfer_step = transfer_diff/frames;
		xRot_step = xRot_diff/frames;
		yRot_step = yRot_diff/frames;
		zRot_step = zRot_diff/frames;
		zoom_step = zoom_diff/frames;

		QString point = "Point Nr. " + QString::number(i) + " [" + QString::number(animPoints[i].samples) + " frames]\n";
		if (writeFile)
			fputs(point, file);

		for (int j = 0; j < frames; j++)
		{
			setRenderStatus("Rendering Magic Lamp Animation " + QString::number(frameCounter+1) + "/" + QString::number(overallFrames));
			QString pointDatas = "   Frame Nr. " + QString::number(frameCounter) + ": " +
				"direction (" + QString::number(direction[0]) + ", " + QString::number(direction[1]) + ", " + QString::number(direction[2]) + "), " +
				"angle (" + QString::number(angle) + "), " + 
				"transfer Factor (" + QString::number(transfer) + "), " +
				"xRot (" + QString::number(xRot) + "), " +
				"yRot (" + QString::number(yRot) + "), " +
				"zRot (" + QString::number(zRot) + "), " +
				"zoom (" + QString::number(zoom) + ")\n";
			if (writeFile)
				fputs(pointDatas, file);

			QString picName;
			if (frameCounter < 10)
				picName = "pic000" + QString::number(frameCounter) + ".jpg";
			else if (frameCounter < 100)
				picName = "pic00" + QString::number(frameCounter) + ".jpg";
			else if (frameCounter < 1000)
				picName = "pic0" + QString::number(frameCounter) + ".jpg";
			else
				picName = "pic" + QString::number(frameCounter) + ".jpg";
			QString filename = directory + "/" + picName;



			// do rendering Step
			float * orientation = new float [3];
			volume->GetOrientation(orientation);
			orientation[0] += xRot_step;
			orientation[1] += yRot_step;
			orientation[2] += zRot_step;
			volume->SetOrientation(orientation);
			volume->SetScale(zoom, zoom, zoom);

			
			// Rotate Lamp Against Object Rotation
			float lampDirection [3];
			lampDirection[0] = direction[0];
			lampDirection[1] = direction[1];
			lampDirection[2] = direction[2];
			float help [3];
			help[0] = lampDirection[0]*cos((-1)*PI/180*orientation[2]) - lampDirection[1]*sin((-1)*PI/180*orientation[2]);
			help[1] = lampDirection[0]*sin((-1)*PI/180*orientation[2]) + lampDirection[1]*cos((-1)*PI/180*orientation[2]);
			lampDirection[0] = help[0];
			lampDirection[1] = help[1];
			help[2] = lampDirection[2]*cos((-1)*PI/180*orientation[1]) - lampDirection[0]*sin((-1)*PI/180*orientation[1]);
			help[0] = lampDirection[2]*sin((-1)*PI/180*orientation[1]) + lampDirection[0]*cos((-1)*PI/180*orientation[1]);
			lampDirection[0] = help[0];
			lampDirection[2] = help[2];
			help[1] = lampDirection[1]*cos((-1)*PI/180*orientation[0]) - lampDirection[2]*sin((-1)*PI/180*orientation[0]);
			help[2] = lampDirection[1]*sin((-1)*PI/180*orientation[0]) + lampDirection[2]*cos((-1)*PI/180*orientation[0]);
			lampDirection[1] = help[1];
			lampDirection[2] = help[2];

			float length = sqrt(lampDirection[0]*lampDirection[0] + lampDirection[1]*lampDirection[1] + lampDirection[2]*lampDirection[2]);
			lampDirection[0] /= length;
			lampDirection[1] /= length;
			lampDirection[2] /= length;

			compositeFunction->setMagicLamp(lampDirection, angle, transfer, backside);

			
			this->Render();

			// save Image
			WindowToImageFilter *w2i = new WindowToImageFilter();
			w2i->SetInput(this);
    
			vtkJPEGWriter *writer = vtkJPEGWriter::New();
			writer->SetInput(w2i->GetOutput ());
			writer->SetFileName(filename);
			writer->Write();

			writer->Delete();
			w2i->Delete();



			// increment values
			direction[0] += direction_step[0];
			direction[1] += direction_step[1];
			direction[2] += direction_step[2];
			angle += angle_step;
			transfer += transfer_step;
			xRot += xRot_step;
			yRot += yRot_step;
			zRot += zRot_step;
			zoom += zoom_step;

			frameCounter += 1;
		}
	}

	fclose(file);

	if (status)
		compositeFunction->setMagicLampEnable();
	else
		compositeFunction->setMagicLampDisable();
	volume->SetOrientation(defaultOrientation);
	volume->SetScale(defaultScale);
	Renderer->BackingStoreOn();
	this->Render();
	setRenderStatus("");
}